<?php

declare(strict_types=1);

namespace Dedi\SyliusContactPlugin\Controller;

use DateTime;
use Dedi\SyliusContactPlugin\Entity\ContactFormMessage;
use Dedi\SyliusContactPlugin\Entity\ContactFormSubjectInterface;
use Dedi\SyliusContactPlugin\Form\ContactFormMessageType;
use Doctrine\ORM\EntityManagerInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Locale\Context\LocaleContextInterface;
use Sylius\Component\Mailer\Sender\SenderInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\Translation\TranslatorInterface;

class ContactFormController extends AbstractController
{
    private FormFactoryInterface $formFactory;
    private EntityManagerInterface $entityManager;
    private TranslatorInterface $translator;
    private SenderInterface $mailer;
    private LocaleContextInterface $localeContext;
    private ChannelContextInterface $channelContext;

    public function __construct(
        FormFactoryInterface $formFactory,
        EntityManagerInterface $entityManager,
        TranslatorInterface $translator,
        SenderInterface $mailer,
        LocaleContextInterface $localeContext,
        ChannelContextInterface $channelContext
    ) {
        $this->formFactory = $formFactory;
        $this->entityManager = $entityManager;
        $this->translator = $translator;
        $this->mailer = $mailer;
        $this->localeContext = $localeContext;
        $this->channelContext = $channelContext;
    }

    public function requestAction(Request $request): Response
    {
        $form = $this->formFactory->create(ContactFormMessageType::class);

        if ($request->isMethod('POST') && $form->handleRequest($request)->isValid()) {
            $data = $form->getData();

            /** @var ContactFormSubjectInterface $subject */
            $subject = $data['subject'];

            $contactMessage = new ContactFormMessage();
            $contactMessage
                ->setSubject($subject)
                ->setName($data['name'])
                ->setEmail($data['email'])
                ->setPhone($data['phone'])
                ->setMessage($data['message'])
                ->setCreatedAt(new DateTime())
                ->setUpdatedAt(new DateTime())
                ->setIp($request->getClientIp());

            $this->entityManager->persist($contactMessage);
            $this->entityManager->flush();

            $mailCode = $this->getSyliusAttribute($request, 'mailCode', 'contact_request_sent');
            $this->mailer->send($mailCode, [$subject->getRecipientEmail()], [
                'contactMessage' => $contactMessage,
                'localeCode' => $this->localeContext->getLocaleCode(),
                'channel' => $this->channelContext->getChannel(),
            ]);

            $this->addFlash('success', $this->translator->trans($this->getSyliusAttribute(
                $request,
                'success_flash',
                'dedi_sylius_contact_form.request_success')
            ));
        }

        $template = $this->getSyliusAttribute(
            $request,
            'template',
            '@DediSyliusContactPlugin/Shop/ContactFormSubject/form.html.twig'
        );

        return $this->render($template, ['form' => $form->createView()]);
    }

    private function getSyliusAttribute(Request $request, string $attributeName, ?string $default): ?string
    {
        $attributes = $request->attributes->get('_sylius');

        return $attributes[$attributeName] ?? $default;
    }
}
